==> fxt-2011.01.04.tgz: fxt/demo/comb/partition-dist-lex-rec-demo.cc <==

// demo-is-self-contained

#include <cmath>

#include "fxttypes.h"

#include "jjassert.h"
#include "demo/nextarg.h"  // NXARG()

#include "fxtio.h"

//% Generate all integer partitions into distinct part in lexicographic order, recursive algorithm.

//#define TIMING // uncomment to disable printing




ulong *a;
ulong ct = 0;

void visit(ulong off)
{
    cout << setw(4) << ++ct << ":  ";
    for (ulong k=0; k<off;  ++k)
    {
        cout << a[k];
        if ( k+1<off )  cout << "+";
    }
//    cout << "  [" << off << "]";  // number of parts
    cout << endl;
}
// -------------------------

void part_dist_lex_rec(ulong n, ulong mp, ulong off=0)
{
    if ( n==0 )
    {
#ifndef TIMING
        visit(off);
#else
        ++ct;
#endif
        return;
    }

    for (ulong m=mp+1; m<=n; ++m)
    {
        a[off] = m;
        part_dist_lex_rec(n-m, m, off+1);
    }
}
// -------------------------

int
main(int argc, char **argv)
{
    ulong n = 9;
    NXARG(n, "Partitions of n");
    const ulong na = (ulong)floor( (sqrt(1+8*(double)n) - 1)/2 );  // max number of parts, see A003056
    a = new ulong[na];
//    cout << " na=" << na << endl;

    ulong mp = 0;
    NXARG(mp, "Minimal part > mp,  0 <= mp < n");
//    if ( mp<1 )  mp = 1;
    if ( mp>=n )  mp = n;

    part_dist_lex_rec(n, mp);

    cout << " " << n << ":  ct=" << ct << endl;
    cout << endl;

    delete [] a;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 180
arg 1: 180 == n  [Partitions of n]  default=9
arg 2: 0 == mp  [Minimal part > mp,  0 <= mp < n]  default=0
 180:  ct=141231780

./bin 180  8.79s user 0.00s system 99% cpu 8.795 total
  ==> 141231780/8.79 == 16,067,324 per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/partition-dist-lex-rec-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/partition-dist-lex-rec-demo.cc DEMOFLAGS=-DTIMING"
/// End:

==> fxt-2011.02.13.tgz: fxt/demo/comb/partition-dist-lex-rec-demo.cc <==

// demo-is-self-contained

#include <cmath>

#include "fxttypes.h"

#include "jjassert.h"
#include "demo/nextarg.h"  // NXARG()

#include "fxtio.h"

//% Generate all integer partitions into distinct part in lexicographic order, recursive algorithm.

//#define TIMING // uncomment to disable printing




ulong *a;
ulong ct = 0;

void visit(ulong off)
{
//    {
////        ulong d1=0;  // OEIS sequences A179254 and A179255
//        ulong d1=a[0];
//        for (ulong k=1; k<off; ++k)
//        {
//            ulong d2 = a[k]-a[k-1];
////            if ( d2<=d1 ) return;  // A179254
////            if ( d2<d1 ) return;  // A179255
////            if ( d2<=d1 ) return;  // A179269
//            if ( d2<d1 ) return;  // A179282
//            d1=d2;
//        }
//    }

    cout << setw(4) << ++ct << ":  ";
    for (ulong k=0; k<off;  ++k)
    {
        cout << a[k];
        if ( k+1<off )  cout << "+";
    }
//    cout << "  [" << off << "]";  // number of parts
    cout << endl;
}
// -------------------------

void part_dist_lex_rec(ulong n, ulong mp, ulong off=0)
{
    if ( n==0 )
    {
#ifndef TIMING
        visit(off);
#else
        ++ct;
#endif
        return;
    }

    const ulong d = 1;  // min distance between parts and minimal part
    // d==1:  A000009
    // d==2:  A003106
    // d==3:  A179046
    // d==4:  N/A
    for (ulong m=mp+d; m<=n; ++m)
    {
        a[off] = m;
        part_dist_lex_rec(n-m, m, off+1);
    }
}
// -------------------------

int
main(int argc, char **argv)
{
    ulong n = 9;
    NXARG(n, "Partitions of n");
    const ulong na = (ulong)floor( (sqrt(1+8*(double)n) - 1)/2 );  // max number of parts, see A003056
    a = new ulong[na];
//    cout << " na=" << na << endl;

    ulong mp = 0;
    NXARG(mp, "Minimal part > mp,  0 <= mp < n");
//    if ( mp<1 )  mp = 1;
    if ( mp>=n )  mp = n;

    part_dist_lex_rec(n, mp);

    cout << " " << n << ":  ct=" << ct << endl;
    cout << endl;

    delete [] a;

    return 0;
}
// -------------------------

/*
Timing:
% time ./bin 180
arg 1: 180 == n  [Partitions of n]  default=9
arg 2: 0 == mp  [Minimal part > mp,  0 <= mp < n]  default=0
 180:  ct=141231780

./bin 180  8.79s user 0.00s system 99% cpu 8.795 total
  ==> 141231780/8.79 == 16,067,324 per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/partition-dist-lex-rec-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/partition-dist-lex-rec-demo.cc DEMOFLAGS=-DTIMING"
/// End:
